from typing import Any, Awaitable, Coroutine, cast, Union, Dict

import grpc
from google.protobuf.json_format import ParseDict
from pycityproto.city.economy.v1 import org_service_pb2 as org_service
from pycityproto.city.economy.v1 import org_service_pb2_grpc as org_grpc
from pycityproto.city.economy.v1 import person_service_pb2 as person_service
from pycityproto.city.economy.v1 import person_service_pb2_grpc as person_grpc

from ..utils.protobuf import async_parse

__all__ = ["EconomyPersonService", "EconomyOrgService"]


class EconomyPersonService:
    """
    城市模拟经济服务（个人）
    City simulation economic service (personal)
    """

    def __init__(self, aio_channel: grpc.aio.Channel):
        self._aio_stub = person_grpc.PersonServiceStub(aio_channel)

    def GetPerson(
        self,
        req: Union[person_service.GetPersonRequest, dict],
        dict_return: bool = True,
    ) -> Coroutine[Any, Any, Union[Dict[str, Any], person_service.GetPersonResponse]]:
        """
        批量查询人的经济情况（资金、雇佣关系）
        Query person’s economic situation (funds, employment relationship) in batches
        
        Args:
        - req (dict): https://cityproto.sim.fiblab.net/#city.economy.v1.GetPersonRequest

        Returns:
        - https://cityproto.sim.fiblab.net/#city.economy.v1.GetPersonResponse
        """
        if type(req) != person_service.GetPersonRequest:
            req = ParseDict(req, person_service.GetPersonRequest())
        res = cast(
            Awaitable[person_service.GetPersonResponse], self._aio_stub.GetPerson(req)
        )
        return async_parse(res, dict_return)

    def UpdatePersonMoney(
        self,
        req: Union[person_service.UpdatePersonMoneyRequest, dict],
        dict_return: bool = True,
    ) -> Coroutine[
        Any, Any, Union[Dict[str, Any], person_service.UpdatePersonMoneyResponse]
    ]:
        """
        批量修改人的资金
        Modify person’s money in batches

        Args:
        - req (dict): https://cityproto.sim.fiblab.net/#city.economy.v1.UpdatePersonMoneyRequest

        Returns:
        - https://cityproto.sim.fiblab.net/#city.economy.v1.UpdatePersonMoneyResponse
        """
        if type(req) != person_service.UpdatePersonMoneyRequest:
            req = ParseDict(req, person_service.UpdatePersonMoneyRequest())
        res = cast(
            Awaitable[person_service.UpdatePersonMoneyResponse],
            self._aio_stub.UpdatePersonMoney(req),
        )
        return async_parse(res, dict_return)


class EconomyOrgService:
    """
    城市模拟经济服务（组织）
    City simulation economic service (organizational)
    """

    def __init__(self, aio_channel: grpc.aio.Channel):
        self._aio_stub = org_grpc.OrgServiceStub(aio_channel)

    def GetOrg(
        self, req: Union[org_service.GetOrgRequest, dict], dict_return: bool = True
    ) -> Coroutine[Any, Any, Union[Dict[str, Any], org_service.GetOrgResponse]]:
        """
        批量查询组织的经济情况（员工、岗位、资金、货物）
        Query the economic status of the organization (employees, positions, funds, goods) in batches

        Args:
        - req (dict): https://cityproto.sim.fiblab.net/#city.economy.v1.GetOrgRequest

        Returns:
        - https://cityproto.sim.fiblab.net/#city.economy.v1.GetOrgResponse
        """
        if type(req) != org_service.GetOrgRequest:
            req = ParseDict(req, org_service.GetOrgRequest())
        res = cast(Awaitable[org_service.GetOrgResponse], self._aio_stub.GetOrg(req))
        return async_parse(res, dict_return)

    def UpdateOrgMoney(
        self,
        req: Union[org_service.UpdateOrgMoneyRequest, dict],
        dict_return: bool = True,
    ) -> Coroutine[Any, Any, Union[Dict[str, Any], org_service.UpdateOrgMoneyResponse]]:
        """
        批量修改组织的资金
        Modify organization’s money in batches

        Args:
        - req (dict): https://cityproto.sim.fiblab.net/#city.economy.v1.UpdateOrgMoneyRequest

        Returns:
        - https://cityproto.sim.fiblab.net/#city.economy.v1.UpdateOrgMoneyResponse
        """
        if type(req) != org_service.UpdateOrgMoneyRequest:
            req = ParseDict(req, org_service.UpdateOrgMoneyRequest())
        res = cast(
            Awaitable[org_service.UpdateOrgMoneyResponse],
            self._aio_stub.UpdateOrgMoney(req),
        )
        return async_parse(res, dict_return)

    def UpdateOrgGoods(
        self,
        req: Union[org_service.UpdateOrgGoodsRequest, dict],
        dict_return: bool = True,
    ) -> Coroutine[Any, Any, Union[Dict[str, Any], org_service.UpdateOrgGoodsResponse]]:
        """
        批量修改组织的货物
        Modify organization’s goods in batches

        Args:
        - req (dict): https://cityproto.sim.fiblab.net/#city.economy.v1.UpdateOrgGoodsRequest

        Returns:
        - https://cityproto.sim.fiblab.net/#city.economy.v1.UpdateOrgGoodsResponse
        """
        if type(req) != org_service.UpdateOrgGoodsRequest:
            req = ParseDict(req, org_service.UpdateOrgGoodsRequest())
        res = cast(
            Awaitable[org_service.UpdateOrgGoodsResponse],
            self._aio_stub.UpdateOrgGoods(req),
        )
        return async_parse(res, dict_return)

    def UpdateOrgEmployee(
        self,
        req: Union[org_service.UpdateOrgEmployeeRequest, dict],
        dict_return: bool = True,
    ) -> Coroutine[
        Any, Any, Union[Dict[str, Any], org_service.UpdateOrgEmployeeResponse]
    ]:
        """
        批量修改组织的员工
        Modify organization’s emplpyees in batches

        Args:
        - req (dict): https://cityproto.sim.fiblab.net/#city.economy.v1.UpdateOrgEmployeeRequest

        Returns:
        - https://cityproto.sim.fiblab.net/#city.economy.v1.UpdateOrgEmployeeResponse
        """
        if type(req) != org_service.UpdateOrgEmployeeRequest:
            req = ParseDict(req, org_service.UpdateOrgEmployeeRequest())
        res = cast(
            Awaitable[org_service.UpdateOrgEmployeeResponse],
            self._aio_stub.UpdateOrgEmployee(req),
        )
        return async_parse(res, dict_return)

    def UpdateOrgJob(
        self,
        req: Union[org_service.UpdateOrgJobRequest, dict],
        dict_return: bool = True,
    ) -> Coroutine[Any, Any, Union[Dict[str, Any], org_service.UpdateOrgJobResponse]]:
        """
        批量修改组织的岗位
        Modify organization’s jobs in batches

        Args:
        - req (dict): https://cityproto.sim.fiblab.net/#city.economy.v1.UpdateOrgJobRequest

        Returns:
        - https://cityproto.sim.fiblab.net/#city.economy.v1.UpdateOrgJobResponse
        """
        if type(req) != org_service.UpdateOrgJobRequest:
            req = ParseDict(req, org_service.UpdateOrgJobRequest())
        res = cast(
            Awaitable[org_service.UpdateOrgJobResponse],
            self._aio_stub.UpdateOrgJob(req),
        )
        return async_parse(res, dict_return)



from typing import Any, Awaitable, Coroutine, cast, Union, Dict

import grpc
from google.protobuf.json_format import ParseDict
from pycityproto.city.map.v2 import aoi_service_pb2 as aoi_service
from pycityproto.city.map.v2 import aoi_service_pb2_grpc as aoi_grpc

from ..utils.protobuf import async_parse

__all__ = ["AoiService"]


class AoiService:
    """
    aoi服务
    AOI service
    """

    def __init__(self, aio_channel: grpc.aio.Channel):
        self._aio_stub = aoi_grpc.AoiServiceStub(aio_channel)

    def GetAoi(
        self, req: Union[aoi_service.GetAoiRequest, dict], dict_return: bool = True
    ) -> Coroutine[Any, Any, Union[Dict[str, Any], aoi_service.GetAoiResponse]]:
        """
        获取AOI信息
        get AOI information
        
        Args:
        - req (dict): https://cityproto.sim.fiblab.net/#city.map.v2.GetAoiRequest

        Returns:
        - https://cityproto.sim.fiblab.net/#city.map.v2.GetAoiResponse
        """
        if type(req) != aoi_service.GetAoiRequest:
            req = ParseDict(req, aoi_service.GetAoiRequest())
        res = cast(Awaitable[aoi_service.GetAoiResponse], self._aio_stub.GetAoi(req))
        return async_parse(res, dict_return)



from typing import Any, Awaitable, Coroutine, cast, Union, Dict

import grpc
from google.protobuf.json_format import ParseDict
from pycityproto.city.clock.v1 import clock_service_pb2 as clock_service
from pycityproto.city.clock.v1 import clock_service_pb2_grpc as clock_grpc

from ..utils.protobuf import async_parse

__all__ = ["ClockService"]


class ClockService:
    """
    城市模拟时间服务
    City simulation clock service
    """

    def __init__(self, aio_channel: grpc.aio.Channel):
        self._aio_stub = clock_grpc.ClockServiceStub(aio_channel)

    def Now(
        self,
        req: Union[clock_service.NowRequest, dict],
        dict_return: bool = True,
    ) -> Coroutine[Any, Any, Union[Dict[str, Any], clock_service.NowResponse]]:
        """
        按照topic查询事件
        Query events by topic
        
        Args:
        - req (dict): https://cityproto.sim.fiblab.net/#city.clock.v1.NowRequest

        Returns:
        - https://cityproto.sim.fiblab.net/#city.clock.v1.NowResponse
        """
        if type(req) != clock_service.NowRequest:
            req = ParseDict(req, clock_service.NowRequest())
        res = cast(
            Awaitable[clock_service.NowResponse],
            self._aio_stub.Now(req),
        )
        return async_parse(res, dict_return)



import logging
from time import sleep
from typing import cast

import grpc
from pycityproto.city.sync.v1 import sync_service_pb2 as sync_service
from pycityproto.city.sync.v1 import sync_service_pb2_grpc as sync_grpc

from ..utils.grpc import create_aio_channel

__all__ = ["OnlyClientSidecar"]


class OnlyClientSidecar:
    """
    Sidecar框架服务（仅支持作为客户端，不支持对外提供gRPC服务）
    Sidecar framework service (only supported as a client, does not support external gRPC services)
    """

    def __init__(self, name: str, syncer_address: str, secure: bool = False):
        """
        Args:
        - name (str): 本服务在etcd上的注册名。The registered name of this service on etcd.
        - server_address (str): syncer地址。syncer address.
        - listen_address (str): sidecar监听地址。sidecar listening address.
        - secure (bool, optional): 是否使用安全连接. Defaults to False. Whether to use a secure connection. Defaults to False.
        """
        self._name = name
        channel = create_aio_channel(syncer_address, secure)
        self._sync_stub = sync_grpc.SyncServiceStub(channel)

    def wait_url(self, name: str) -> str:
        """
        获取服务的uri
        Get the uri of the service

        Args:
        - name (str): 服务的注册名。Service registration name.

        Returns:
        - str: 服务的url。service url.
        """
        while True:
            try:
                resp = cast(
                    sync_service.GetURLResponse,
                    self._sync_stub.GetURL(sync_service.GetURLRequest(name=name)),
                )
                url = resp.url
                break
            except grpc.RpcError as e:
                logging.warning("get uri failed, retrying..., %s", e)
                sleep(1)

        logging.debug("get uri: %s for name=%s", url, name)
        return url

    def step(self, step: int) -> bool:
        """
        同步器步进
        synchronizer step up

        Args:
        - step (int): 同步器步进步数。number of synchronizer step up.

        Returns:
        - close (bool): 是否退出模拟。Whether the simulation exited.
        """
        request = sync_service.StepRequest(name=self._name, step=step)
        response = self._sync_stub.Step(request)
        return response.close

    def init(self) -> bool:
        """
        同步器初始化
        Synchronizer initialization

        Returns:
        - close (bool): 是否退出模拟。Whether the simulation exited.

        Examples:
        ```python
        close = client.init()
        print(close)
        # > False
        ```
        """
        return self.step(1)

    def close(self) -> bool:
        """
        同步器关闭
        Synchronizer close
        
        Returns:
        - close (bool): 是否退出模拟。Whether the simulation exited.
        """
        return self.step(-1)

    def notify_step_ready(self):
        """
        通知prepare阶段已完成
        Notify that the prepare phase is completed
        """
        ...



from typing import Any, Awaitable, Coroutine, cast, Union, Dict

import grpc
from google.protobuf.json_format import ParseDict
from pycityproto.city.event.v2 import event_service_pb2 as event_service
from pycityproto.city.event.v2 import event_service_pb2_grpc as event_grpc

from ..utils.protobuf import async_parse

__all__ = ["EventService"]


class EventService:
    """
    城市模拟事件服务
    City simulation event service
    """

    def __init__(self, aio_channel: grpc.aio.Channel):
        self._aio_stub = event_grpc.EventServiceStub(aio_channel)

    def GetEventsByTopic(
        self,
        req: Union[event_service.GetEventsByTopicRequest, dict],
        dict_return: bool = True,
    ) -> Coroutine[
        Any, Any, Union[Dict[str, Any], event_service.GetEventsByTopicResponse]
    ]:
        """
        按照topic查询事件
        Query event by topic

        Args:
        - req (dict): https://cityproto.sim.fiblab.net/#city.event.v2.GetEventsByTopicRequest

        Returns:
        - https://cityproto.sim.fiblab.net/#city.event.v2.GetEventsByTopicResponse
        """
        if type(req) != event_service.GetEventsByTopicRequest:
            req = ParseDict(req, event_service.GetEventsByTopicRequest())
        res = cast(
            Awaitable[event_service.GetEventsByTopicResponse],
            self._aio_stub.GetEventsByTopic(req),
        )
        return async_parse(res, dict_return)

    def ResolveEvents(
        self,
        req: Union[event_service.ResolveEventsRequest, dict],
        dict_return: bool = True,
    ) -> Coroutine[
        Any, Any, Union[Dict[str, Any], event_service.ResolveEventsResponse]
    ]:
        """
        确认事件已被处理
        Confirm that the event has been processed
        
        Args:
        - req (dict): https://cityproto.sim.fiblab.net/#city.event.v2.ResolveEventsRequest

        Returns:
        - https://cityproto.sim.fiblab.net/#city.event.v2.ResolveEventsResponse
        """
        if type(req) != event_service.ResolveEventsRequest:
            req = ParseDict(req, event_service.ResolveEventsRequest())
        res = cast(
            Awaitable[event_service.ResolveEventsResponse],
            self._aio_stub.ResolveEvents(req),
        )
        return async_parse(res, dict_return)



from typing import Any, Awaitable, TypeVar, Union, Dict
from google.protobuf.message import Message
from google.protobuf.json_format import MessageToDict

__all__ = ["parse", "async_parse"]

T = TypeVar("T", bound=Message)


def parse(res: T, dict_return: bool) -> Union[Dict[str, Any], T]:
    """
    将Protobuf返回值转换为dict或者原始值
    Convert Protobuf return value to dict or original value
    """
    if dict_return:
        return MessageToDict(
            res,
            including_default_value_fields=True,
            preserving_proto_field_name=True,
            use_integers_for_enums=True,
        )
    else:
        return res


async def async_parse(res: Awaitable[T], dict_return: bool) -> Union[Dict[str, Any], T]:
    """
    将Protobuf await返回值转换为dict或者原始值
    Convert Protobuf await return value to dict or original value
    """
    if dict_return:
        return MessageToDict(
            await res,
            including_default_value_fields=True,
            preserving_proto_field_name=True,
            use_integers_for_enums=True,
        )
    else:
        return await res
